#ifndef FGGL_DATA_MODEL_H
#define FGGL_DATA_MODEL_H

#include <vector>
#include <fggl/math/types.hpp>

namespace fggl::data {

	struct Vertex {
		math::vec3 posititon;
		math::vec3 normal;
		math::vec3 colour;
	};

	class Mesh {
		public:
			Mesh();
			~Mesh() = default;

			/**
			 * Push a vertex with deduplication.
			 *
			 * If the vertex is already known, the old vertex will be used and a new entry will
			 * be added to the indicies list.
			 */
			inline void push(Vertex vert) {
				int idx = indexOf(vert);
				if ( idx == -1 )
					pushVertex(vert);
				else
					pushIndex(idx);
			}

			/**
			 * Push a new vertex index.
			 *
			 * Don't push non-existant indexes, it'll make the rendering backend sad.
			 */
			void pushIndex(unsigned int idx);

			/**
			 * Force push a vertex duplicate.
			 *
			 * If the vetex already exists, it will be duplicated.
			 */
			unsigned int pushVertex(Vertex vert);


			/**
			 * Search for a vertex.
			 *
			 * Returns the index of a vert, else returns -1.
			 */
			int indexOf(Vertex vert);

			inline std::size_t vertexCount() {
				return m_verts.size();
			}

			inline std::size_t indexCount() {
				return m_index.size();
			}

			inline Vertex& vertex(int idx) {
				return m_verts[idx];
			}

		private:
			std::vector<Vertex> m_verts;
			std::vector<unsigned int> m_index;
	};

	class Model {
		public:
			Model();
			~Model();

			inline void append(const Mesh& mesh) {
				m_meshes.push_back( mesh );
			}

			inline const std::vector<Mesh>& meshes() const {
				return m_meshes;
			}

		private:
			std::vector<Mesh> m_meshes;
	};

}

#endif
